<?php

/*
 * Global settings for boards.
 */

class EveryBoard_Global_Settings {

    private $framework;
    private $custom_css_classes;
    private $active_engine;

    /**
     * Settings constructor, adds actions and sets private member.s
     * @param string $file Path of file that called constructor.
     */
    public function __construct( $file ) {

        global $pagenow;

        $options                    = get_option( 'render_framework' );
        $this->framework            = isset( $options['framework'] ) ? $options['framework'] : '';
        $classes                    = get_option( 'board_custom_css_classes' ) ? get_option( 'board_custom_css_classes' ) : array();
        $this->custom_css_classes   = $classes;

        if ( is_admin() ) {
            add_action( 'admin_menu', array( &$this, 'register_board_settings_submenu_page' ) );
            add_action( 'admin_init', array( &$this, 'register_board_global_settings' ) );

            if( $pagenow === 'edit.php' && isset( $_GET['page'] ) && $_GET['page'] === 'boardsettings' ) {
                add_action( 'admin_enqueue_scripts', array( &$this, 'eb_global_settings_enqueue_scripts' ) );
            }
        }

        // Ajax Actions
        add_action( 'wp_ajax_eb_global_settings_ajax_get_css_classes', array( &$this, 'eb_global_settings_ajax_get_css_classes' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_add_css_class', array( &$this, 'eb_global_settings_ajax_add_css_class' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_delete_css_class', array( &$this, 'eb_global_settings_ajax_delete_css_class' ) );
        add_action( 'wp_ajax_eb_global_settings_ajax_get_css_classes_import', array( &$this, 'eb_global_settings_ajax_get_css_classes_import' ) );

        if( isset( $file ) && $file !== '') {
            register_activation_hook( $file, array( &$this, 'eb_global_settings_add_defaults' ) );
        }
    }

    /**
     * Execute action enqueue scripts, includes needed scripts and stylesheets for this page.
     */
    public function eb_global_settings_enqueue_scripts() {
        wp_enqueue_style( 'widgetprefab-style', EVERYBOARD_BASE . 'assets/dist/css/boardsettings.min.css' );
        wp_enqueue_script( 'eb_global_settings_js', EVERYBOARD_BASE . 'assets/dist/js/boardsettings.min.ugly.js' );
        wp_localize_script('eb_global_settings_js', 'translation_boardsettings', array(
            'remove_class_confirm' => __("Are you sure you want to remove this class?", "everyboard"),
            'remove_class_failure' => __("Could not remove class, please try again or contact an administrator.", "everyboard"),
            'class_import_failure' => __("Import failed, please check the format and try again.", "everyboard"),
            'add_class_failure' => __("There was a problem adding the class, please try again.", "everyboard"),
            'class_name_not_valid' => __('CSS Class name not valid, please try again. ( Only alphanumeric characters and "-" and "_" are allowed ).', "everyboard")
        ));
    }

    /**
     * Registers the settings for boards.
     */
    public function register_board_global_settings() {
        register_setting( 'everyboard-global-settings', 'render_framework' );
    }

    /**
     * AJAX FUNCTION - Prints the existing custom css classes.
     */
    public function eb_global_settings_ajax_get_css_classes() {
        print json_encode( $this->eb_global_settings_get_css_classes() );
        die();
    }

    /**
     * AJAX FUNCTION - Adds a given CSS class to the settings.
     * @param string $css_class The class to add.
     * @return string either "success" or "fail" depending on status.
     */

    public function eb_global_settings_ajax_add_css_class() {

        $css_name   = $_POST['css_name'];
        $css_class  = $_POST['css_class'];
        $class_row  = isset( $_POST['row_class'] ) ? $_POST['row_class'] : false;
        $class_col  = isset( $_POST['col_class'] ) ? $_POST['col_class'] : false;
        $class_art  = isset( $_POST['art_class'] ) ? $_POST['art_class'] : false;
        $class_wid  = isset( $_POST['wid_class'] ) ? $_POST['wid_class'] : false;
        $unique_id  = isset( $_POST['id'] ) ? $_POST['id'] : uniqid( 'class_', false );

        $class_arr      = array(
            'name'    => $css_name,
            'class'   => $css_class,
            'row'     => $class_row,
            'column'  => $class_col,
            'article' => $class_art,
            'widget'  => $class_wid,
            'id'      => $unique_id
        );

        $insert_array   = $this->eb_global_settings_get_css_classes();
        array_push( $insert_array, $class_arr );
        //$insert_array = array();

        if( update_option( 'board_custom_css_classes', $insert_array ) ) {
            $this->custom_css_classes = $insert_array;
            print $unique_id;
        }
        else {
            print 'false';
        }

        die();
    }

    public function eb_global_settings_ajax_delete_css_class() {

        if( isset( $_POST['classId'] ) ) {

            $classId    = $_POST['classId'];
            $del_array  = $this->eb_global_settings_get_css_classes();
            $delIndex   = null;

            for( $i = 0; $i < count( $del_array ); $i++ ) {

                if( $del_array[$i]['id'] === $classId ) {
                    $delIndex = $i;
                }
            }

            if( $delIndex !== null ) {
                array_splice( $del_array, $delIndex, 1 );
            }

            if( update_option( 'board_custom_css_classes', $del_array ) ) {
                $this->custom_css_classes = $del_array;
                print 'success';
            }
            else {
                print 'false';
            }
        }
        else {
            print 'false';
        }

        die();
    }

    /**
     * Returns existing CSS classes.
     * @return array List of current classes name of class is key and the actual class is the value.
     */
    public function eb_global_settings_get_css_classes() {
        return $this->custom_css_classes;
    }

    /**
     * Registers menu item for settings page, a submenu page of "Boards".
     */
    public function register_board_settings_submenu_page() {
        add_submenu_page( 'edit.php?post_type=everyboard', 'Board Settings', __('Board Settings', 'everyboard'), 'manage_options', 'boardsettings', array( &$this, 'admin_page' ) );
    }

    /**
     * Adds default values for board settings, runs on activation of plugin.
     */
    public function eb_global_settings_add_defaults() {

        $eb_framework_helper = new EveryBoard_Framework_Helper( false );
        update_option( 'render_framework', array(
            'framework' => $eb_framework_helper->eb_default_framework,
        ) );
    }

    /**
     * Get a list of current CSS classes formatted for export / import.
     */
    public function eb_global_settings_ajax_get_css_classes_export() {

        $classes    = $this->eb_global_settings_get_css_classes();
        $ret_str    = '';

        foreach ($classes as $key => $value) {
            $ret_str .= $key . "|" . $value . "\n";
        }

        print $ret_str;

        die();
    }

    /**
     * Imports a list of CSS classes.
     */
    public function eb_global_settings_ajax_get_css_classes_import() {

        if( isset( $_POST['importData'] ) ) {

            try {

                $import_data    = json_decode( stripslashes( $_POST['importData'] ) );
                $insert_array   = $this->eb_global_settings_get_css_classes();

                //old to new import
                if( !isset( $import_data ) ) {
                    $import_data = $_POST['importData'];
                    $lines = array_filter( explode(PHP_EOL, $import_data) );


                    foreach( $lines as $item ) {
                        $itemparts = explode( '|', $item );

                        $class_arr = array(
                            'name'   => $itemparts[1],
                            'class'  => $itemparts[0],
                            'row'    => 'true',
                            'column' => 'true',
                            'article'=> 'true',
                            'widget' => 'true',
                            'id'     => uniqid( 'class_', false )
                        );

                        array_push( $insert_array, $class_arr );
                    }
                } else {
                    foreach ( $import_data as $item ) {

                        $class_arr      = array(
                            'name'    => $item->name,
                            'class'   => $item->class,
                            'row'     => $item->row,
                            'column'  => $item->column,
                            'article' => $item->article,
                            'widget'  => $item->widget,
                            'id'      => uniqid( 'class_', false )
                        );

                        array_push( $insert_array, $class_arr );
                    }
                }

                if( update_option( 'board_custom_css_classes', $insert_array ) ) {
                    print __('success', 'everyboard');
                }
                else {
                    print __('false', 'everyboard');
                }
            }
            catch( Exception $e ) {
                print __('false', 'everyboard');
            }
        }
        else {
            print __('false', 'everyboard');
        }


        die();
    }

    /**
     * Renders the actual options page for board settings.
     */
    function admin_page() {

        $eb_framework_helper    = new EveryBoard_Framework_Helper( false );
        $frameworks             = $eb_framework_helper->get_frameworks();
        $active_framework       = esc_attr( $this->framework );

        # Start HTML ?>
        <div class="wrap">
            <h2><?php echo __('EveryBoard Settings', 'everyboard'); ?></h2>

                <?php settings_errors(); ?>

                <form method="post" action="options.php">
                <?php settings_fields( 'everyboard-global-settings' ); ?>

                <table class="form-table">

                    <tr valign="top"><td colspan="2"><h3><?php echo __('Board Rendering Framework', 'everyboard'); ?></h3></td></tr>

                    <tr valign="top">

                        <th scope="row">
                            <label for="board_settings_framework_selector"><?php echo __('Rendering Framework:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Choose the framework to use when rendering the board data. Normally this should be the same framework that your theme is built on (if any).', 'everyboard'); ?></p>
                        </th>

                        <td>

                            <select id="board_settings_framework_selector" name="render_framework[framework]">

                                <?php foreach( $frameworks as $slug => $framework ) {
                                    $selected = ( $active_framework === $slug ) ? 'selected="selected"' : ''; ?>

                                    <option value="<?= $slug; ?>" <?= $selected; ?>> <?= $framework; ?></option>

                                <?php } ?>

                            </select>

                        </td>
                    </tr>

                    <tr valign="top"><td colspan="2"><h3><?php echo __('Board Custom CSS Classes', 'everyboard'); ?></h3></td></tr>

                    <tr valign="top" class="table_row_extra_bottom_space">

                        <th scope="row">
                            <label for="board_settings_css_classes_add_name_input"><?php echo __('Add board custom CSS class:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Add a new CSS class that will be available when editing an board.', 'everyboard'); ?></p>
                        </th>

                        <td>
                            <label for="board_settings_css_classes_add_name_input"><?php echo __('Class Name', 'everyboard'); ?> <span class="description">(<?php echo __('the exact name of the actual css class', 'everyboard'); ?>):</span></label><br />
                            <input type="text" name="" value="" id="board_settings_css_classes_add_name_input" /><br />

                            <label for="board_settings_css_classes_add_class_input"><?php echo __('Class Description', 'everyboard'); ?> <span class="description">(<?php echo __('a description of the class for the end user', 'everyboard'); ?>):</span></label><br />
                            <input type="text" name="" value="" id="board_settings_css_classes_add_class_input" /><br />

                            <div id="class-type-container">
                                <?php echo __('Accessible to items of type:', 'everyboard'); ?>
                                <table class="class-type-table board-settings-table">
                                    <tr>
                                        <td><?php echo __('Row:', 'everyboard'); ?></td>
                                        <td><input type="checkbox" name="row-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><?php echo __('Column:', 'everyboard'); ?></td>
                                        <td><input type="checkbox" name="col-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><?php echo __('Article:', 'everyboard'); ?></td>
                                        <td><input type="checkbox" name="art-class" /></td>
                                    </tr>
                                    <tr>
                                        <td><?php echo __('Widget:', 'everyboard'); ?></td>
                                        <td><input type="checkbox" name="wid-class" /></td>
                                    </tr>
                                </table>
                            </div>

                            <a href="#" class="button" id="board_settings_css_classes_add_button"><?php echo __('Add class', 'everyboard'); ?></a>
                        </td>
                    </tr>

                    <tr valign="top">
                        <th scope="row">
                            <label for="board_settings_css_classes_list"><?php echo __('Current board custom CSS classes:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('The CSS classes that are available to users when editing a board.', 'everyboard'); ?></p>
                        </th>
                        <td>
                            <table id="board_settings_css_classes_table" class="board-settings-table">
                                <tr>
                                    <th><?php echo __('Class name', 'everyboard'); ?></th>
                                    <th><?php echo __('Description', 'everyboard'); ?></th>
                                    <th><?php echo __('For types', 'everyboard'); ?></th>
                                </tr>
                            </table>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="board_settings_css_classes_export_area"><?php echo __('Export / Import board custom CSS classes:', 'everyboard'); ?></label>
                            <p class="description"><?php echo __('Paste CSS classes from another site or copy the existing ones to export to another site.', 'everyboard'); ?></p>
                        </th>
                        <td>
                            <textarea class="board_settings_export_area" id="board_settings_css_classes_export_area" rows="10" cols="100"></textarea>
                            <p>
                                <a href="#" class="button" id="board_settings_css_classes_import_button"><?php echo __('Import CSS Classes', 'everyboard'); ?></a>
                                <a href="#" class="button" id="board_settings_css_classes_export_button"><?php echo __('Export CSS Classes', 'everyboard'); ?></a>
                            </p>
                        </td>
                    </tr>

                </table>
                <?php submit_button(); ?>
            </form>
        </div>

        <?php # End HTML
    }
}